#ifndef __R7_H
#define __R7_H
#include "orbitersdk.h"
#include "multi_stage.h"

const double LENGTH_STAGE0        = 20.8;
const double LENGTH_STAGE1        = 23.82;
const double LENGTH_NOSECONE      =  3.8;
const double LENGTH_COWLING       =  0.44;

const VECTOR3 OFS_STAGE1        = { 0.0     , 0.0     , 0.0};
const VECTOR3 OFS_STAGE0_LEFT   = { -1.3-1.2, 0.0     , OFS_STAGE1.z-(LENGTH_STAGE1-LENGTH_STAGE0) / 2};
const VECTOR3 OFS_STAGE0_RIGHT  = {  1.3+1.2, 0.0     , OFS_STAGE0_LEFT.z};
const VECTOR3 OFS_STAGE0_TOP    = {  0.0    ,  1.3+1.2, OFS_STAGE0_LEFT.z};
const VECTOR3 OFS_STAGE0_BOTTOM = {  0.0    , -1.3-1.2, OFS_STAGE0_LEFT.z};
const VECTOR3 OFS_NOSECONE      = { 0.0     , 0.0     , OFS_STAGE1.z + LENGTH_STAGE1 / 2 + LENGTH_NOSECONE / 2};
const VECTOR3 OFS_COWLING       = { 0.0     , 0.0     , OFS_NOSECONE.z + LENGTH_NOSECONE / 2 + LENGTH_COWLING / 2};

const VECTOR3 OFS_BOOSTER_LEFT_FLAME   = {-1.3-0.8,      0.0, OFS_STAGE1.z-LENGTH_STAGE1/2};
const VECTOR3 OFS_BOOSTER_RIGHT_FLAME  = { 1.3+0.8,      0.0, OFS_STAGE1.z-LENGTH_STAGE1/2};
const VECTOR3 OFS_BOOSTER_TOP_FLAME    = {     0.0, 1.3+0.8 , OFS_STAGE1.z-LENGTH_STAGE1/2};
const VECTOR3 OFS_BOOSTER_BOTTOM_FLAME = {     0.0, -1.3-0.8, OFS_STAGE1.z-LENGTH_STAGE1/2};
//const VECTOR3 OFS_STAGE0_RIGHT_FLAME = {OFS_STAGE0_RIGHT.x+.2, 0.0,  OFS_STAGE0_RIGHT.z-14.0};
const VECTOR3 OFS_STAGE1_FLAME = { 0.0, 0.0,  OFS_STAGE1.z-LENGTH_STAGE1/2};

const double MASS_FIRST    = 95300*KG;
const double MASS_BOOSTER  = 43100*KG;
const double MASS_COWLING  = 10*KG;

const double MASS_FIRST_EMPTY   = 7500*KG;
const double MASS_FIRST_FUEL    = MASS_FIRST-MASS_FIRST_EMPTY;
const double MASS_BOOSTER_EMPTY = 3500*KG;
const double MASS_BOOSTER_FUEL  = MASS_BOOSTER-MASS_BOOSTER_EMPTY;


const double ISP_BOOSTER_SL  = 250*SEC;
const double ISP_BOOSTER_VAC = 306*SEC;
const double ISP_FIRST_SL    = 241*SEC;
const double ISP_FIRST_VAC   = 308*SEC;

const double THRUST_BOOSTER_VAC = 99000*KGF;
const double THRUST_FIRST_VAC   = 93000*KGF;

VECTOR3 sat_ofs              = _V(0,0,0);
VECTOR3 ofs_sts_sat          = _V(0,0,0);
VECTOR3 nosecone_ofs         = _V(0,0,0);
char sat_name[256]           ="";
char sat_mesh_name[256]      ="";
char adapter_mesh_name[256]  ="";
char nosecone_mesh_name[256] ="";
bool sat_jettisoned          =true;
double sat_mass	             = 0;

void AddLeftBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("r7booster1", &OFS_STAGE0_LEFT);
};


void AddRightBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("r7booster2", &OFS_STAGE0_RIGHT);
};

void AddTopBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("r7booster3", &OFS_STAGE0_TOP);
};


void AddBottomBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("r7booster4", &OFS_STAGE0_BOTTOM);
};

void AddCowling(VESSEL *vessel)
{
	vessel->AddMesh ("r7cowling", &OFS_COWLING);
};


void AddFirstStage(VESSEL *vessel)
{
	vessel->AddMesh ("r7body", &OFS_STAGE1);
};

void AddNoseCone(VESSEL *vessel)
{
	vessel->AddMesh ("r7nosecone", &OFS_NOSECONE);
};

void AddStageOneExhaust(VESSEL *vessel)
{
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	const double OFFSET = .25;
	const double LENGTH = 5;
	const double WIDTH = .15;
	mesh_dir=_V(0,0,-1);

	mesh_pos=OFS_STAGE1_FLAME ;
	mesh_pos.x += OFFSET;
	mesh_pos.y += OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=OFS_STAGE1_FLAME ;
	mesh_pos.x -= OFFSET;
	mesh_pos.y += OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=OFS_STAGE1_FLAME ;
	mesh_pos.x += OFFSET;
	mesh_pos.y -= OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=OFS_STAGE1_FLAME ;
	mesh_pos.x -= OFFSET;
	mesh_pos.y -= OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);
};

void AddBoosterExhaust(VESSEL *vessel, VECTOR3 booster_pos)
{
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	const double OFFSET = .25;
	const double LENGTH = 5;
	const double WIDTH = .15;
	mesh_dir=_V(0,0,-1);

	mesh_pos=booster_pos ;
	mesh_pos.x += OFFSET;
	mesh_pos.y += OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=booster_pos ;
	mesh_pos.x -= OFFSET;
	mesh_pos.y += OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=booster_pos ;
	mesh_pos.x += OFFSET;
	mesh_pos.y -= OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);

	mesh_pos=booster_pos;
	mesh_pos.x -= OFFSET;
	mesh_pos.y -= OFFSET;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,LENGTH, WIDTH,&mesh_dir);
};


void AddStageOneJets(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	const double ATTCOOR = 0.9;
	const double ATTZ = OFS_STAGE1_FLAME.z;
	const double ATTWIDTH=.2;
	const double ATTHEIGHT=.5;

	att_dir = _V(0,0,-1);
	//up
	att_ref= _V(0,-ATTCOOR,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(0,-ATTCOOR,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);


	//dn
	att_ref= _V(0,ATTCOOR,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref= _V(0,ATTCOOR,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	

	//right
	att_ref= _V(-ATTCOOR,0,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(-ATTCOOR,0,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	//left
	att_ref= _V(ATTCOOR,0,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(ATTCOOR,0,ATTZ);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
}
void AddBoosterJets(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	const double  OFFLENGTH = .9;
	const double  OFFWIDTH = .3;
	const double ATTWIDTH=.2;
	const double ATTHEIGHT=.5;

	att_dir = _V(0,0,-1);
	//up
	att_ref= OFS_BOOSTER_BOTTOM_FLAME;
	att_ref.x += OFFWIDTH;
	att_ref.y -= OFFLENGTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= OFS_BOOSTER_BOTTOM_FLAME;
	att_ref.x -= OFFWIDTH;
	att_ref.y -= OFFLENGTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);


	//dn
	att_ref= OFS_BOOSTER_TOP_FLAME;
	att_ref.x += OFFWIDTH;
	att_ref.y += OFFLENGTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref= OFS_BOOSTER_TOP_FLAME;
	att_ref.x -= OFFWIDTH;
	att_ref.y += OFFLENGTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	

	//right
	att_ref= OFS_BOOSTER_LEFT_FLAME;
	att_ref.x -= OFFLENGTH;
	att_ref.y += OFFWIDTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= OFS_BOOSTER_LEFT_FLAME;
	att_ref.x -= OFFLENGTH;
	att_ref.y -= OFFWIDTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	//left
	att_ref= OFS_BOOSTER_RIGHT_FLAME;
	att_ref.x += OFFLENGTH;
	att_ref.y += OFFWIDTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= OFS_BOOSTER_RIGHT_FLAME;
	att_ref.x += OFFLENGTH;
	att_ref.y -= OFFWIDTH;
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
}
class R7FirstStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonSatellite(VESSEL *vessel);
	void JettisonShroud(VESSEL *vessel);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
	bool ShroudJettisoned;
	double empty_mass;
};

void R7FirstStage::Init ()
{
	bManualSeparate = false;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
	ShroudJettisoned=false;
	empty_mass      = MASS_FIRST_EMPTY + MASS_COWLING;
};
void R7FirstStage::Exit (VESSEL *vessel)
{
};
void R7FirstStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};
void R7FirstStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	int jettisoned=0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
	double readdouble = 0.0;
 	if (!strnicmp (line, "SAT_JETTISONED", 14)) {
        sscanf (line+14, "%d", &jettisoned);
		if (jettisoned==1) {
			sat_jettisoned=true;
		} else {
			sat_jettisoned=false;
		}
	} else if (!strnicmp (line, "SAT_OFS_X", 9)) {
		sscanf (line+9, "%lf", &sts_sat_x);
		ofs_sts_sat.x=sts_sat_x;
	} else if (!strnicmp (line, "SAT_OFS_Y", 9)) {
		sscanf (line+9, "%lf", &sts_sat_y);
		ofs_sts_sat.y=sts_sat_y;
	} else if (!strnicmp (line, "SAT_OFS_Z", 9)) {
		sscanf (line+9, "%lf", &sts_sat_z);
		ofs_sts_sat.z=sts_sat_z;
	} else if (!strnicmp (line, "SAT_NAME", 8)) {
		sscanf (line+8, "%s", sat_name);
	} else if (!strnicmp (line, "SAT_MESH", 8)) {
		sscanf (line+8, "%s", sat_mesh_name);
	} else if (!strnicmp (line, "SAT_MASS", 8)) {
		sscanf (line+8, "%lf", &sat_mass);
	} else if (!strnicmp (line, "ADAPTER_NAME", 12)) {
		sscanf (line+12, "%s", adapter_mesh_name);
	} else if (!strnicmp (line, "NOSECONE_NAME", 13)) {
		sscanf (line+13, "%s", nosecone_mesh_name);
	} else if (!strnicmp (line, "SHROUD", 6)) {
		sscanf (line+6, "%d", &jettisoned);
		if (jettisoned==1) {
			ShroudJettisoned=true;
		} else {
			ShroudJettisoned=false;
		}
	} else if (!strnicmp (line, "NOSECONE_OFS_X", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.x=readdouble;
	} else if (!strnicmp (line, "NOSECONE_OFS_Y", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.y=readdouble;
	} else if (!strnicmp (line, "NOSECONE_OFS_Z", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.z=readdouble;
	} else {
		vessel->ParseScenarioLine (line, def_vs);
	}
};
void R7FirstStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	int jettisoned = 0;
	oapiWriteScenario_int (scn, "CONFIGURATION", 1);
	oapiWriteScenario_float (scn, "SAT_OFS_X", ofs_sts_sat.x);
	oapiWriteScenario_float (scn, "SAT_OFS_Y", ofs_sts_sat.y);
	oapiWriteScenario_float (scn, "SAT_OFS_Z", ofs_sts_sat.z);
	oapiWriteScenario_float (scn, "SAT_MASS", sat_mass);
	if (sat_jettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SAT_JETTISONED", jettisoned);
	oapiWriteScenario_string (scn, "SAT_NAME", sat_name);
	oapiWriteScenario_string (scn, "ADAPTER_NAME", adapter_mesh_name);
	oapiWriteScenario_string (scn, "NOSECONE_NAME", nosecone_mesh_name);
	oapiWriteScenario_string (scn, "SAT_MESH", sat_mesh_name);
	if (ShroudJettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SHROUD", jettisoned);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_X", nosecone_ofs.x);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_Y", nosecone_ofs.y);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_Z", nosecone_ofs.z);
};
void R7FirstStage::Timestep (VESSEL *vessel, double simt)
{
	OBJHANDLE hvessel;
	if (vessel->GetFuelMass() == 0 || bManualSeparate)
	{
		bManualSeparate=false;
		if (ShroudJettisoned)
		{
			if (!sat_jettisoned)
			{
				JettisonSatellite(vessel);
				sat_jettisoned=true;
				SetStage(vessel);
				vessel->SetEngineLevel(ENGINE_MAIN,0);
				char name[256];
				strcpy (name, vessel->GetName()); strcat (name, "-Sat");
				hvessel=oapiGetVesselByName(name);
				if (hvessel != 0)
					oapiSetFocusObject(hvessel);
			}
		}
		else
		{
			JettisonShroud(vessel);
			ShroudJettisoned=true;
			SetStage(vessel);
		}
		return;
	}
	DoThrustAdjust(vessel,slThrust,vacThrust);
};
void R7FirstStage::Animate (VESSEL *vessel, double simt)
{
};
void R7FirstStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void R7FirstStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int R7FirstStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 0;
		}
	}
	return 0;
};

void R7FirstStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_FIRST_FUEL);
	vessel->SetEmptyMass  (empty_mass);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e3);

	//vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
	//vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z), _V(-.7,.7,OFS_STAGE1_FLAME.z), _V(.7,.7,OFS_STAGE1_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddStageOneExhaust(vessel);
	AddStageOneJets(vessel);
	AddFirstStage(vessel);
	AddNoseCone(vessel);
	if (!ShroudJettisoned)
	{
		AddCowling(vessel);
	}
	if (!sat_jettisoned)
	{
		vessel->AddMesh(sat_mesh_name,&ofs_sts_sat);
		vessel->SetEmptyMass(vessel->GetEmptyMass()+sat_mass);
	}
};

void R7FirstStage::JettisonSatellite(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0.25);
	ofs = ofs_sts_sat;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);

	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Sat");
	VESSEL::Create (name, sat_name, vs);
};

void R7FirstStage::JettisonShroud(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0.0, 0, 1.0);
	ofs = OFS_COWLING;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_Cowling");
	VESSEL::Create (name, "R7_Cowling", vs);
};

class R7BoosterStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void doBoosterTimeStep (VESSEL *vessel, double simt);
	bool bManualSeparate;
	double slThrust;
	double vacThrust;
	double slBooster;
	double vacBooster;
	double last_simt;
	double booster_fuel;
	double empty_mass;
	int boost_id1;
	int boost_id2;
	int nBoosters;
};

void R7BoosterStage::Init ()
{
	bManualSeparate = false;
	nBoosters       = 4;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
	slBooster       = CalcSeaLevelThrust(THRUST_BOOSTER_VAC,ISP_BOOSTER_VAC,ISP_BOOSTER_SL,MASS_BOOSTER_FUEL);
	vacBooster      = THRUST_BOOSTER_VAC;
	last_simt       = 0.0;
	empty_mass      = MASS_BOOSTER_EMPTY * nBoosters + MASS_FIRST_EMPTY + MASS_COWLING;
	booster_fuel    = MASS_BOOSTER_FUEL * nBoosters;
};

void R7BoosterStage::Exit (VESSEL *vessel)
{
};

void R7BoosterStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};

void R7BoosterStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	int jettisoned=0;
	double old_ignition_time=0.0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
	double readdouble = 0.0;
 	if (!strnicmp (line, "BOOSTER_FUEL", 12)) {
        sscanf (line+12, "%lf", &booster_fuel);
 	} else if (!strnicmp (line, "SAT_JETTISONED", 14)) {
        sscanf (line+14, "%d", &jettisoned);
		if (jettisoned==1) {
			sat_jettisoned=true;
		} else {
			sat_jettisoned=false;
	}
	} else if (!strnicmp (line, "SAT_OFS_X", 9)) {
		sscanf (line+9, "%lf", &sts_sat_x);
		ofs_sts_sat.x=sts_sat_x;
	} else if (!strnicmp (line, "SAT_OFS_Y", 9)) {
		sscanf (line+9, "%lf", &sts_sat_y);
		ofs_sts_sat.y=sts_sat_y;
	} else if (!strnicmp (line, "SAT_OFS_Z", 9)) {
		sscanf (line+9, "%lf", &sts_sat_z);
		ofs_sts_sat.z=sts_sat_z;
	} else if (!strnicmp (line, "SAT_NAME", 8)) {
		sscanf (line+8, "%s", sat_name);
	} else if (!strnicmp (line, "SAT_MESH", 8)) {
		sscanf (line+8, "%s", sat_mesh_name);
	} else if (!strnicmp (line, "SAT_MASS", 8)) {
		sscanf (line+8, "%lf", &sat_mass);
	} else if (!strnicmp (line, "ADAPTER_NAME", 12)) {
		sscanf (line+12, "%s", adapter_mesh_name);
	} else if (!strnicmp (line, "NOSECONE_NAME", 13)) {
		sscanf (line+13, "%s", nosecone_mesh_name);
	} else if (!strnicmp (line, "NOSECONE_OFS_X", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.x=readdouble;
	} else if (!strnicmp (line, "NOSECONE_OFS_Y", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.y=readdouble;
	} else if (!strnicmp (line, "NOSECONE_OFS_Z", 14)) {
		sscanf (line+14, "%lf", &readdouble);
		nosecone_ofs.z=readdouble;
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }
};

void R7BoosterStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	int jettisoned = 0;
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	oapiWriteScenario_float (scn, "BOOSTER_FUEL", booster_fuel);
	oapiWriteScenario_float (scn, "SAT_OFS_X", ofs_sts_sat.x);
	oapiWriteScenario_float (scn, "SAT_OFS_Y", ofs_sts_sat.y);
	oapiWriteScenario_float (scn, "SAT_OFS_Z", ofs_sts_sat.z);
	if (sat_jettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SAT_JETTISONED", jettisoned);
	oapiWriteScenario_string (scn, "SAT_NAME", sat_name);
	oapiWriteScenario_string (scn, "SAT_MESH", sat_mesh_name);
	oapiWriteScenario_string (scn, "ADAPTER_NAME", adapter_mesh_name);
	oapiWriteScenario_string (scn, "NOSECONE_NAME", nosecone_mesh_name);
	oapiWriteScenario_float (scn, "SAT_MASS", sat_mass);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_X", nosecone_ofs.x);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_Y", nosecone_ofs.y);
	oapiWriteScenario_float (scn, "NOSECONE_OFS_Z", nosecone_ofs.z);
};
void R7BoosterStage::Timestep (VESSEL *vessel, double simt)
{
	double FuelLevel=0.0;
	IStage *newStage;
	
	if ((booster_fuel <= 0.0) || bManualSeparate)
	{
		bManualSeparate=false;
		JettisonStage(vessel);
		newStage = new R7FirstStage();
		Stack->SetStage(newStage,vessel,simt);
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);
		return;
	};
	doBoosterTimeStep(vessel,simt);
	last_simt=simt;
};

void R7BoosterStage::doBoosterTimeStep (VESSEL *vessel, double simt)
{
	double *booster_thrust;
	double c_booster_thrust=0.0;
	double booster_time=0.0;
	booster_thrust=vessel->GetMainThrustModPtr();	
	*booster_thrust = 0.0;
	booster_fuel -=GetFuelUsage(oapiGetSimStep(),vacBooster*nBoosters*vessel->GetEngineLevel(ENGINE_MAIN),ISP_BOOSTER_VAC);
	if (booster_fuel <= 0)
		booster_fuel = 0;
	if (booster_fuel > 0) 
	{
		c_booster_thrust = GetCurrentValue(vessel,slBooster,vacBooster)*nBoosters*vessel->GetEngineLevel(ENGINE_MAIN);
		vessel->SetEmptyMass(empty_mass+booster_fuel);
	}
	//vessel->SetMaxThrust(ENGINE_RETRO,GetFuelUsage(oapiGetSimStep(),c_booster_thrust,ISP_BOOSTER_VAC));
	//vessel->SetMaxThrust(ENGINE_HOVER,booster_fuel);
	*booster_thrust = (GetCurrentValue(vessel,slThrust,vacThrust)-vacThrust)+c_booster_thrust;
}

void R7BoosterStage::Animate (VESSEL *vessel, double simt)
{
};
void R7BoosterStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void R7BoosterStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int R7BoosterStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 0;
		}
	}
	return 0;
};

void R7BoosterStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_FIRST_FUEL);
	vessel->SetEmptyMass  (empty_mass + booster_fuel*nBoosters);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e3);

	//vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
	//vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z), _V(-.7,.7,OFS_STAGE1_FLAME.z), _V(.7,.7,OFS_STAGE1_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddStageOneExhaust(vessel);
	AddBoosterExhaust(vessel, OFS_BOOSTER_LEFT_FLAME);
	AddBoosterExhaust(vessel, OFS_BOOSTER_RIGHT_FLAME);
	AddBoosterExhaust(vessel, OFS_BOOSTER_TOP_FLAME);
	AddBoosterExhaust(vessel, OFS_BOOSTER_BOTTOM_FLAME);
	AddStageOneJets(vessel);
	AddBoosterJets(vessel);
	AddLeftBoosterStage(vessel);
	AddRightBoosterStage(vessel);
	AddTopBoosterStage(vessel);
	AddBottomBoosterStage(vessel);
	AddFirstStage(vessel);
	AddNoseCone(vessel);
	AddCowling(vessel);
	if (!sat_jettisoned)
	{
		vessel->AddMesh (sat_mesh_name,&ofs_sts_sat);
		vessel->SetEmptyMass(vessel->GetEmptyMass()+sat_mass);
	}
};

void R7BoosterStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(-2.0,0,0);
	ofs = OFS_STAGE0_LEFT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_LeftBooster");
	VESSEL::Create (name, "R7_LeftBooster", vs);

	vel = _V(2.0,0,0);
	ofs = OFS_STAGE0_RIGHT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_RightBooster");
	VESSEL::Create (name, "R7_RightBooster", vs);

  	vel = _V(0,2.0,0);
	ofs = OFS_STAGE0_TOP;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_TopBooster");
	VESSEL::Create (name, "R7_TopBooster", vs);

	vel = _V(0,-2.0,0);
	ofs = OFS_STAGE0_BOTTOM;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_BottomBooster");
	VESSEL::Create (name, "R7_BottomBooster", vs);

 };

#endif