#ifndef __SPUTNIK_H
#define __SPUTNKI_H
#include "orbitersdk.h"
#include "multi_stage.h"
#include "windows.h"

const double MASS_SPUTNIK  = 84*KG;
const VECTOR3 OFS_SPUTNIK  = _V(0,0,0);


void AddSputnik(VESSEL *vessel)
{
	vessel->AddMesh ("sputnik", &OFS_SPUTNIK);
};


class Sputnik:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	bool bRadio;
	double empty_mass;
};

void Sputnik::Init ()
{
	empty_mass      = MASS_SPUTNIK;
	bRadio = false;
};

void Sputnik::Exit (VESSEL *vessel)
{
};

void Sputnik::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};

void Sputnik::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	int doRadio=0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
 	if (!strnicmp (line, "RADIO", 5)) {
        sscanf (line+5, "%d", &doRadio);
		if (doRadio==1) {
			bRadio=true;
		} else {
			bRadio=false;
		}
	} else {
		vessel->ParseScenarioLine (line, def_vs);
	}
};
void Sputnik::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	int doRadio = 0;
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	if (bRadio)
		doRadio=1;
	else
		doRadio=0;
	oapiWriteScenario_int (scn, "SAT_JETTISONED", doRadio);
};

void Sputnik::Timestep (VESSEL *vessel, double simt)
{
};

void Sputnik::Animate (VESSEL *vessel, double simt)
{
};

void Sputnik::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};

void Sputnik::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};

int Sputnik::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_O)) { // "operate radio"
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
			{
				if (bRadio)
				{
					PlaySound(NULL , NULL, NULL);
					bRadio=false;
					vessel->SetMaxThrust(ENGINE_RETRO,0);
				}
				else
				{
					PlaySound("sputnik.wav",NULL, SND_FILENAME | SND_ASYNC | SND_NODEFAULT | SND_LOOP);
					vessel->SetMaxThrust(ENGINE_RETRO,1);
					bRadio=true;
				}
			}
			return 1;
		}

		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 0;
		}
	}
	return 0;
};

void Sputnik::SetStage(VESSEL *vessel)
{
	vessel->SetSize (1.0);
	vessel->SetCOG_elev (.1);
	vessel->SetMaxFuelMass(.01);
	vessel->SetEmptyMass  (empty_mass);
	vessel->SetISP        (0);
	vessel->SetMaxThrust  (ENGINE_MAIN, 0);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e2);

	//vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
	//vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	//vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z), _V(-.7,.7,OFS_STAGE1_FLAME.z), _V(.7,.7,OFS_STAGE1_FLAME.z));
	vessel->SetCW (0.1, 0.1, 0.1, 0.1);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddSputnik(vessel);
};



#endif